@@ -1,7 +1,13 @@
 """Constants used throughout the project."""
 
+from typing import TypeVar
+
 from .enums import TaskGroup
 
+# Type variable used for generic typing
+T = TypeVar("T", bound=object)
+
+
 # This is used as input to generative models; it cannot be a special token
 DUMMY_FILL_VALUE = 100
 
@@ -50,9 +56,11 @@ METRIC_ATTRIBUTES_TAKING_UP_MEMORY = ["cached_bertscorer"]
 # Hugging Face Hub tags used to classify models as merge models
 MERGE_TAGS = ["merge", "mergekit"]
 
+
 # The minimum required CUDA compute capability for using bfloat16 in vLLM
 VLLM_BF16_MIN_CUDA_COMPUTE_CAPABILITY = 8.0
 
+
 # Used to detect whether a model is a reasoning model
 REASONING_TOKENS = [
     ("<think>", "</think>"),
@@ -60,6 +68,7 @@ REASONING_TOKENS = [
     ("<reasoning>", "</reasoning>"),
 ]
 
+
 # These tokens are sometimes used by models to indicate the end of a generated
 # response, but they do not use them as a proper EOS token, so we have to deal with them
 # manually. We only use them as stop tokens if they actually appear in the model's
