@@ -26,6 +26,7 @@ MAX_REPETITIONS = 50
 
 
 CHOICES_MAPPING = {
+    "cs": "Výběr",
     "da": "Svarmuligheder",
     "de": "Antwortmöglichkeiten",
     "en": "Choices",
@@ -36,10 +37,12 @@ CHOICES_MAPPING = {
     "fr": "Choix",
     "is": "Svarmöguleikar",
     "it": "Scelte",
+    "lt": "Pasirinkimai",
     "lv": "Izvēles",
     "nl": "Antwoordopties",
     "no": "Svaralternativer",
     "pl": "Opcje",
     "pt": "Opções",
+    "sk": "Možnosti",
     "sv": "Svarsalternativ",
 }
