@@ -22,6 +22,7 @@ from datasets.arrow_dataset import Dataset
 from datasets.dataset_dict import DatasetDict
 from huggingface_hub.hf_api import HfApi
 from load_ud_pos import (
+    load_csdt_pos,
     load_dadt_pos,
     load_dedt_pos,
     load_endt_pos,
@@ -32,12 +33,14 @@ from load_ud_pos import (
     load_frdt_pos,
     load_isdt_pos,
     load_itdt_pos,
+    load_ltdt_pos,
     load_lvdt_pos,
     load_nldt_pos,
     load_nodt_nb_pos,
     load_nodt_nn_pos,
     load_pldt_pos,
     load_ptdt_pos,
+    load_skdt_pos,
     load_svdt_pos,
 )
 from pandas.errors import SettingWithCopyWarning
@@ -65,6 +68,9 @@ def main() -> None:
         "lv": load_lvdt_pos,
         "et": load_etdt_pos,
         "pl": load_pldt_pos,
+        "lt": load_ltdt_pos,
+        "cs": load_csdt_pos,
+        "sk": load_skdt_pos,
     }
 
     # Set up the progress bar and iterate over the languages
