@@ -11,7 +11,9 @@
 import logging
 import re
 from collections import defaultdict
-from typing import Callable, Dict, List, Union
+from functools import partial
+from typing import Callable, DefaultDict
+from urllib.parse import urlparse
 
 import pandas as pd
 import requests
@@ -20,7 +22,7 @@ logging.basicConfig(format="%(asctime)s ⋅ %(message)s", level=logging.INFO)
 logger = logging.getLogger("load_ud_pos")
 
 
-def load_dadt_pos() -> Dict[str, pd.DataFrame]:
+def load_dadt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Danish Dependency Treebank.
 
     Returns:
@@ -38,7 +40,7 @@ def load_dadt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_ptdt_pos() -> Dict[str, pd.DataFrame]:
+def load_ptdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Portuguese Dependency Treebank.
 
     Returns:
@@ -61,7 +63,7 @@ def load_ptdt_pos() -> Dict[str, pd.DataFrame]:
     )
 
 
-def load_fodt_pos() -> Dict[str, pd.DataFrame]:
+def load_fodt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Faroese Dependency Treebank.
 
     Returns:
@@ -79,7 +81,7 @@ def load_fodt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_isdt_pos() -> Dict[str, pd.DataFrame]:
+def load_isdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Icelandic Dependency Treebank.
 
     Returns:
@@ -97,7 +99,7 @@ def load_isdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_nodt_nb_pos() -> Dict[str, pd.DataFrame]:
+def load_nodt_nb_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Norwegian Bokmål Dependency Treebank.
 
     Returns:
@@ -115,7 +117,7 @@ def load_nodt_nb_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_nodt_nn_pos() -> Dict[str, pd.DataFrame]:
+def load_nodt_nn_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Norwegian Nynorsk Dependency Treebank.
 
     Returns:
@@ -133,7 +135,7 @@ def load_nodt_nn_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_svdt_pos() -> Dict[str, pd.DataFrame]:
+def load_svdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Swedish Dependency Treebank.
 
     Returns:
@@ -180,7 +182,7 @@ def load_svdt_pos() -> Dict[str, pd.DataFrame]:
     )
 
 
-def load_dedt_pos() -> Dict[str, pd.DataFrame]:
+def load_dedt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the German Dependency Treebank.
 
     Returns:
@@ -198,7 +200,7 @@ def load_dedt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_nldt_pos() -> Dict[str, pd.DataFrame]:
+def load_nldt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Dutch Dependency Treebank.
 
     Returns:
@@ -216,7 +218,7 @@ def load_nldt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_endt_pos() -> Dict[str, pd.DataFrame]:
+def load_endt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the English Dependency Treebank.
 
     Returns:
@@ -234,7 +236,7 @@ def load_endt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_frdt_pos() -> Dict[str, pd.DataFrame]:
+def load_frdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the French Dependency Treebank.
 
     Returns:
@@ -252,7 +254,7 @@ def load_frdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_itdt_pos() -> Dict[str, pd.DataFrame]:
+def load_itdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Italian Dependency Treebank.
 
     Returns:
@@ -270,7 +272,7 @@ def load_itdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_esdt_pos() -> Dict[str, pd.DataFrame]:
+def load_esdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Spanish Dependency Treebank.
 
     Returns:
@@ -288,7 +290,7 @@ def load_esdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_fidt_pos() -> Dict[str, pd.DataFrame]:
+def load_fidt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Finnish Dependency Treebank.
 
     Returns:
@@ -306,7 +308,7 @@ def load_fidt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_lvdt_pos() -> Dict[str, pd.DataFrame]:
+def load_lvdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Latvian Dependency Treebank.
 
     Returns:
@@ -324,7 +326,7 @@ def load_lvdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_etdt_pos() -> Dict[str, pd.DataFrame]:
+def load_etdt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Estonian Dependency Treebank.
 
     Returns:
@@ -342,7 +344,7 @@ def load_etdt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
-def load_pldt_pos() -> Dict[str, pd.DataFrame]:
+def load_pldt_pos() -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of the Polish Dependency Treebank.
 
     Returns:
@@ -360,13 +362,195 @@ def load_pldt_pos() -> Dict[str, pd.DataFrame]:
     return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
 
 
+def load_ltdt_pos() -> dict[str, pd.DataFrame]:
+    """Load the part-of-speech part of the Lithuanian Dependency Treebank.
+
+    Returns:
+        The dataframes, stored in the keys `train`, `val` and `test`.
+    """
+    # Define download URLs
+    base_url = (
+        "https://raw.githubusercontent.com/UniversalDependencies/UD_Lithuanian-ALKSNIS/"
+        "master/lt_alksnis-ud-{}.conllu"
+    )
+    train_url = base_url.format("train")
+    val_url = base_url.format("dev")
+    test_url = base_url.format("test")
+
+    return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
+
+
+def load_csdt_pos() -> dict[str, pd.DataFrame]:
+    """Load the part-of-speech part of the Czech Dependency Treebank.
+
+    Returns:
+        The dataframes, stored in the keys `train`, `val` and `test`.
+    """
+    # Define download URLs
+    base_url = (
+        "https://raw.githubusercontent.com/UniversalDependencies/UD_Czech-CAC/refs/"
+        "heads/master/cs_cac-ud-{}.conllu"
+    )
+    train_url = base_url.format("train")
+    val_url = base_url.format("dev")
+    test_url = base_url.format("test")
+
+    return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
+
+
+def load_skdt_pos() -> dict[str, pd.DataFrame]:
+    """Load the part-of-speech part of the Slovak Dependency Treebank.
+
+    Returns:
+        The dataframes, stored in the keys `train`, `val`, and `test`.
+    """
+    # Define download URLs
+    base_url = (
+        "https://raw.githubusercontent.com/UniversalDependencies/UD_Slovak-SNK/refs/heads/master/"
+        "sk_snk-ud-{}.conllu"
+    )
+    train_url = base_url.format("train")
+    val_url = base_url.format("dev")
+    test_url = base_url.format("test")
+
+    return load_ud_pos(train_url=train_url, val_url=val_url, test_url=test_url)
+
+
+def _load_file_or_url(url_or_path: str) -> list[str]:
+    """Load a file from a URL or local path.
+
+    Args:
+        url_or_path: The URL or local path to load.
+
+    Returns:
+        The list of strings, one per line.
+    """
+    parsed = urlparse(url_or_path)
+    if parsed.scheme.lower() in ("http", "https"):
+        return requests.get(url_or_path).text.split("\n")
+    else:
+        with open(url_or_path, "r") as f:
+            logger.warning(f"Loading data from local file: {url_or_path}")
+            return [line.strip() for line in f.readlines()]
+
+
+def _filter_token_range(data_dict: dict[str, list]) -> dict[str, list]:
+    """Filter out tokens that belong to ranges in UD source files.
+
+    Tokens that span more than one position are not supported by create_scala's
+    prepare_df logic.
+
+    Example files:
+
+    - tests/test_scripts/test_create_scala/test_data/de_gsd-ud-train.conllu.adp_det
+    - tests/test_scripts/test_create_scala/test_data/pl_pdb-ud-train.conllu.aux_clitic_*
+
+    Args:
+        data_dict: The input data dictionary.
+
+    Returns:
+        The filtered data dictionary. Its format is identical to the input.
+    """
+    output: DefaultDict[str, list] = defaultdict(list)
+
+    range_start: int = 0
+    range_end: int = 0
+
+    # Pattern used to detect merged IDs
+    merged_ids_pattern = re.compile(r"(\d+)-(\d+)", re.I)
+
+    for i in range(len(data_dict["ids"])):
+        match = merged_ids_pattern.match(data_dict["ids"][i])
+        if match is not None:
+            output["ids"].append(data_dict["ids"][i])
+            output["tokens"].append(data_dict["tokens"][i])
+            output["pos_tags"].append(data_dict["pos_tags"][i])
+            range_start = int(match.group(1))
+            range_end = int(match.group(2))
+        else:
+            token_id = int(data_dict["ids"][i].split(".")[0])
+            if token_id >= range_start and token_id <= range_end:
+                # Skip token if in range
+                continue
+            else:
+                output["ids"].append(data_dict["ids"][i])
+                output["tokens"].append(data_dict["tokens"][i])
+                output["pos_tags"].append(data_dict["pos_tags"][i])
+
+    return output
+
+
+def _load_split(
+    *,
+    lines: list[str],
+    filter_source: str | None = None,
+    doc_process_fn: Callable[[str], str] = lambda x: x,
+) -> pd.DataFrame:
+    """Load single split of the POS part of a Universal Dependencies treebank.
+
+    Args:
+        lines: The lines of the file to process.
+        filter_source:
+            If not `None`, only include entries with this source in the dataset.
+        doc_process_fn:
+            A function to apply to each document before parsing it.
+
+    Returns:
+        The dataframe for the given split.
+    """
+    # Initialise the records, data dictionary and document
+    records: list[dict] = []
+    data_dict: dict[str, list[list[int | str] | str]] = defaultdict(list)
+
+    # Iterate over the data for the given split
+    doc = ""
+    source = ""
+    for line_idx, line in enumerate(lines):
+        # If we are at the first line of an entry then extract the document
+        if line.startswith("# text = "):
+            doc = re.sub("# text = ", "", line)
+
+            # Process the document if needed
+            doc = doc_process_fn(doc)
+
+        elif line.startswith("# source = "):
+            source = line.removeprefix("# source = ").strip()
+
+        # Otherwise, if the line is a comment then ignore it
+        elif line.startswith("#"):
+            continue
+
+        # Otherwise, if we have reached the end of an entry then store it to the
+        # list of records and reset the data dictionary and document
+        elif line == "" or line_idx == len(lines) - 1:
+            if len(data_dict["tokens"]) > 0:
+                if filter_source is None or filter_source in source:
+                    merged_data_dict: dict[str, str | list[int | str]]
+                    merged_data_dict = {**_filter_token_range(data_dict), "doc": doc}
+                    records.append(merged_data_dict)
+            data_dict = defaultdict(list)
+            doc = ""
+            source = ""
+
+        # Otherwise we are in the middle of an entry which is not a comment, so
+        # we extract the data from the line and store it in the data dictionary
+        else:
+            data_tup = line.split("\t")
+            data_dict["ids"].append(data_tup[0])
+            data_dict["tokens"].append(data_tup[1])
+            data_dict["pos_tags"].append(data_tup[3])
+
+    # Convert the records to a dataframe
+    return pd.DataFrame.from_records(records)
+
+
 def load_ud_pos(
     train_url: str,
     val_url: str,
     test_url: str,
     doc_process_fn: Callable[[str], str] = lambda x: x,
     filter_source: str | None = None,
-) -> Dict[str, pd.DataFrame]:
+) -> dict[str, pd.DataFrame]:
     """Load the part-of-speech part of a Universal Dependencies treebank.
 
     Args:
@@ -384,65 +568,25 @@ def load_ud_pos(
     Returns:
         The dataframes, stored in the keys `train`, `val` and `test`.
     """
-    # Download the data
-    data = dict(
-        train=requests.get(train_url).text.split("\n"),
-        val=requests.get(val_url).text.split("\n"),
-        test=requests.get(test_url).text.split("\n"),
-    )
-
     if filter_source is not None:
         logger.warning(
             f"Warning: Filtering dataset to include only entries with {filter_source=}"
         )
 
-    # Iterate over the data splits
-    dfs = dict()
-    for split, lines in data.items():
-        # Initialise the records, data dictionary and document
-        records = list()
-        data_dict: Dict[str, List[Union[int, str]]] = defaultdict(list)
-        doc = ""
-        source = ""
+    # Load the lines of the splits
+    train_lines = _load_file_or_url(url_or_path=train_url)
+    val_lines = _load_file_or_url(url_or_path=val_url)
+    test_lines = _load_file_or_url(url_or_path=test_url)
 
-        # Iterate over the data for the given split
-        for line in lines:
-            # If we are at the first line of an entry then extract the document
-            if line.startswith("# text = "):
-                doc = re.sub("# text = ", "", line)
-
-                # Process the document if needed
-                doc = doc_process_fn(doc)
-
-            elif line.startswith("# source = "):
-                source = line.removeprefix("# source = ").strip()
-
-            # Otherwise, if the line is a comment then ignore it
-            elif line.startswith("#"):
-                continue
-
-            # Otherwise, if we have reached the end of an entry then store it to the
-            # list of records and reset the data dictionary and document
-            elif line == "":
-                if len(data_dict["tokens"]) > 0:
-                    if filter_source is None or filter_source in source:
-                        merged_data_dict: Dict[str, Union[str, List[Union[int, str]]]]
-                        merged_data_dict = {**data_dict, "doc": doc}
-                        records.append(merged_data_dict)
-                data_dict = defaultdict(list)
-                doc = ""
-                source = ""
-
-            # Otherwise we are in the middle of an entry which is not a comment, so
-            # we extract the data from the line and store it in the data dictionary
-            else:
-                data_tup = line.split("\t")
-                data_dict["ids"].append(data_tup[0])
-                data_dict["tokens"].append(data_tup[1])
-                data_dict["pos_tags"].append(data_tup[3])
-
-        # Convert the records to a dataframe and store it
-        dfs[split] = pd.DataFrame.from_records(records)
+    # Load the splits
+    split_loader = partial(
+        _load_split, filter_source=filter_source, doc_process_fn=doc_process_fn
+    )
+    dfs = dict(
+        train=split_loader(lines=train_lines),
+        val=split_loader(lines=val_lines),
+        test=split_loader(lines=test_lines),
+    )
 
     # Return the dictionary of dataframes
     return dfs
