@@ -1,13 +1,16 @@
 """General fixtures used throughout test modules."""
 
+import collections.abc as c
 import os
 import sys
 from typing import Generator
 
 import pytest
 import torch
+from click import ParamType
 
-from euroeval.data_models import BenchmarkConfig, ModelConfig, Task
+from euroeval.cli import benchmark
+from euroeval.data_models import BenchmarkConfig, DatasetConfig, ModelConfig, Task
 from euroeval.dataset_configs import SPEED_CONFIG, get_all_dataset_configs
 from euroeval.enums import InferenceBackend, ModelType
 from euroeval.languages import DA, get_all_languages
@@ -18,7 +21,7 @@ from euroeval.tasks import SENT, SPEED, get_all_tasks
 def pytest_configure() -> None:
     """Set a global flag when `pytest` is being run."""
     setattr(sys, "_called_from_test", True)
-    os.environ["CUDA_VISIBLE_DEVICES"] = "0"  # Ensure only one GPU is used in tests
+    os.environ["CUDA_VISIBLE_DEVICES"] = "-1"  # Ensure no GPUs are used in tests
 
 
 def pytest_unconfigure() -> None:
@@ -26,15 +29,18 @@ def pytest_unconfigure() -> None:
     delattr(sys, "_called_from_test")
 
 
-ACTIVE_LANGUAGES = {
-    language_code: language
-    for language_code, language in get_all_languages().items()
-    if any(
-        language in cfg.languages
-        for cfg in get_all_dataset_configs().values()
-        if cfg != SPEED_CONFIG
-    )
-}
+if os.environ.get("TEST_ALL_LANGUAGES", "0") == "1":
+    ACTIVE_LANGUAGES = {
+        language_code: language
+        for language_code, language in get_all_languages().items()
+        if any(
+            language in cfg.languages
+            for cfg in get_all_dataset_configs().values()
+            if cfg != SPEED_CONFIG
+        )
+    }
+else:
+    ACTIVE_LANGUAGES = dict(da=DA)
 
 
 @pytest.fixture(scope="session")
@@ -88,11 +94,12 @@ def benchmark_config(
         num_iterations=1,
         api_base=None,
         api_version=None,
-        gpu_memory_utilization=0.9,
+        gpu_memory_utilization=0.8,
         generative_type=None,
         debug=False,
         run_with_cli=True,
         requires_safetensors=False,
+        download_only=False,
     )
 
 
@@ -173,3 +180,22 @@ def model_config() -> Generator[ModelConfig, None, None]:
         model_cache_dir="cache_dir",
         adapter_base_model_id=None,
     )
+
+
+@pytest.fixture(scope="module")
+def cli_params() -> Generator[dict[str | None, ParamType], None, None]:
+    """Yields a dictionary of the CLI parameters."""
+    ctx = benchmark.make_context(info_name="testing", args=["--model", "test-model"])
+    yield {p.name: p.type for p in benchmark.get_params(ctx)}
+
+
+@pytest.fixture(scope="session")
+def dataset_config() -> c.Generator[DatasetConfig, None, None]:
+    """Yields a dataset configuration used in tests."""
+    yield DatasetConfig(
+        name="dataset",
+        pretty_name="Dataset",
+        huggingface_id="dataset_id",
+        task=SENT,
+        languages=[DA],
+    )
